// v1.4
// unidade que trata da classe TAndar
Unit Andares ;

interface

uses Classes       ,
     Salas         ,
     LuzesSala     ,
     LuzesCorredor ,
     Constantes    ;

type
   TAndar = class
   private
      _sala        : TList                      ; // lista de salas mapeadas
      _corredor    : array[ 1..3 ] of TCorredor  ; // as sees de corredor
      _intensidade : Integer                    ; // a iluminao ambiente
      _direcao     : Integer                    ; // a direo desta iluminao
      _falha       : Boolean                    ; // indicao de falha no SSA
      _sensorLuz   : array[ 1..6 ] of TSensorLuz ; // sensores de luz externo
   public
      constructor     Create                                    ;
      destructor      Destroy                                   ; override  ;
      function        obterEnergia( var iEnerPass : integer )   : Integer   ;
      procedure       simularFalha;
      function        obterSala( pvsala : string )               : TSala      ;
      function        obterCorredor( pvcorredor : String )       : TCorredor  ;
      class procedure iniciarSimulacao ( var poAndar: TAndar )       ;
      procedure       incIntensidadeAmbiente                    ;
      procedure       decIntensidadeAmbiente                    ;
      procedure       mudarDirecaoAmbiente( direcao : Integer ) ;
      function        obterDirecaoAmbiente                      : Integer   ;
      function        obterIntensidadeAmbiente                  : Integer   ;
      function        obterSensorLuz( sensorLuz : Integer )     : TSensorLuz ;
      function        obterFalha                                : Boolean   ;
   end ; // TAndar


implementation

// cria e inicializa o andar
constructor TAndar.Create ;
var
   i         : Integer ;
   SensorAux : TList   ;
begin
   _direcao := 360 ;   // a direo  inicializada com 360 graus (= 0)
   _intensidade := 0 ; // a intensidade da iluminao ambiente comea sendo 0
   _falha := false ;   // o sinal de sistema de controle ativo est funcionando

// cria os 6 sensores externos de luz
// i = 1: ols1; i = 2: ols2; i = 3: ols3; i = 4: ols4; i = 5: ols5; i = 6: ols6
   for i := 1 to 6 do
   begin
      _sensorLuz[ i ].Create( ) ; // cria o sensor externo de luz olsi
   end ; // for

   _sala := TList.Create( ) ; // inicializa a lista de salas

// existem 22 salas contabilizadas, das quais 20 so controladas pelo sistema
// para cada sala controlada:
// 1-  inicializada uma lista auxiliar de sensores externos de luz que a mapeia
// 2- a essa lista so adcionados os sensores externos de luz que a mapeia
// 3- esta sala criada com seu nome, essa lista de sensores e o nmero de portas
// 4- esta sala  adcionada a lista de sala do andar
// 5- esta sala  adcionada na lista de salas dos sensores de luz que a mapeia

// HL410
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 6 ] ) ;
   _sala.Add( TSala.Create( 'HL410', SensorAux , 3 ) ) ;
   _sensorLuz[ 6 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// HL411
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 5 ] ) ;
   _sala.Add( TSala.Create( 'HL411', SensorAux, 2 ) ) ;
   _sensorLuz[ 5 ].AdcionarSala( TSala( _sala.Items[ _sala.Count - 1  ] ) ) ;

// O412
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 4 ] ) ;
   SensorAux.Add( _sensorLuz[ 6 ] ) ;
   _sala.Add( TSala.Create( 'O412', SensorAux, 1 ) ) ;
   _sensorLuz[ 4 ].AdcionarSala( TSala( _sala.Items[ _sala.Count - 1 ] ) ) ;
   _sensorLuz[ 6 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// P413 no  controlada pelo sistema de controle de luz

// O414
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 4 ] ) ;
   _sala.Add( TSala.Create( 'O414', SensorAux, 2 ) ) ;
   _sensorLuz[ 4 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// O415
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 3 ] ) ;
   _sala.Add( TSala.Create( 'O415', SensorAux, 1 ) ) ;
   _sensorLuz[ 3 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// O416
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 4 ] ) ;
   _sala.Add( TSala.Create( 'O416', SensorAux, 3 ) ) ;
   _sensorLuz[ 4 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// O417
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 3 ] ) ;
   _sala.Add( TSala.Create( 'O417', SensorAux, 1 ) ) ;
   _sensorLuz[ 3 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// CL418
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 4 ] ) ;
   _sala.Add( TSala.Create( 'CL418', SensorAux, 3 ) ) ;
   _sensorLuz[ 4 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// O419
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 3 ] ) ;
   _sala.Add( TSala.Create( 'O419', SensorAux, 1 ) ) ;
   _sensorLuz[ 3 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// P420 no  controlada pelo sistema de controle de luz

// O421
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 3 ] ) ;
   SensorAux.Add( _sensorLuz[ 5 ] ) ;
   _sala.Add( TSala.Create( 'O421', SensorAux, 2 ) ) ;
   _sensorLuz[ 3 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;
   _sensorLuz[ 5 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// CL422
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 2 ] ) ;
   SensorAux.Add( _sensorLuz[ 3 ] ) ;
   _sala.Add( TSala.Create( 'CL422', SensorAux, 2 ) ) ;
   _sensorLuz[ 2 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;
   _sensorLuz[ 3 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// O423
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 5 ] ) ;
   _sala.Add( TSala.Create( 'O423', SensorAux, 3 ) ) ;
   _sensorLuz[ 5 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// O424
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 2 ] ) ;
   _sala.Add( TSala.Create( 'O424', SensorAux, 1 ) ) ;
   _sensorLuz[ 2 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// O425
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 5 ] ) ;
   _sala.Add( TSala.Create( 'O425', SensorAux, 2 ) ) ;
   _sensorLuz[ 5 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// CL426
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 2 ] ) ;
   _sala.Add( TSala.Create( 'CL426', SensorAux, 1 ) ) ;
   _sensorLuz[ 2 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// M427
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 1 ] ) ;
   SensorAux.Add( _sensorLuz[ 5 ] ) ;
   _sala.Add( TSala.Create( 'M427', SensorAux, 2 ) ) ;
   _sensorLuz[ 1 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;
   _sensorLuz[ 5 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// O429
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 1 ] ) ;
   _sala.Add( TSala.Create( 'O429', SensorAux, 1 ) ) ;
   _sensorLuz[ 1 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// O431
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 1 ] ) ;
   _sala.Add( TSala.Create( 'O431', SensorAux, 2 ) ) ;
   _sensorLuz[ 1 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// O433
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 1 ] ) ;
   _sala.Add( TSala.Create( 'O433', SensorAux, 3 ) ) ;
   _sensorLuz[ 1 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;

// O435
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 1 ] ) ;
   _sala.Add( TSala.Create( 'O435', SensorAux, 2 ) ) ;
   _sensorLuz[ 1 ].AdcionarSala( TSala( _sala.Items[ _sala.Count -1 ] ) ) ;


// existem 3 sees de corredores, e para cada seo
// 1-  inicializada uma lista auxiliar de sensores externos de luz que a mapeia
// 2- a essa lista so adcionados os sensores externos de luz que a mapeia
// 3- esta seo  criada com seu nome e essa lista de sensores
// 4- esta seo de corredor  colocada no sensor externo de luz que a mapeia

// H1
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 1 ] ) ;
   SensorAux.Add( _sensorLuz[ 2 ] ) ;
   _corredor[ 1 ].Create( 'H1', SensorAux ) ;
   _sensorLuz[ 1 ].AdcionarCorredor( _corredor[ 1 ] ) ;
   _sensorLuz[ 2 ].AdcionarCorredor( _corredor[ 1 ] ) ;

// H2
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 3 ] ) ;
   SensorAux.Add( _sensorLuz[ 4 ] ) ;
   _corredor[ 2 ].Create( 'H2', SensorAux ) ;
   _sensorLuz[ 3 ].AdcionarCorredor( _corredor[ 2 ] ) ;
   _sensorLuz[ 4 ].AdcionarCorredor( _corredor[ 2 ] ) ;

// H3
   SensorAux := TList.Create( ) ;
   SensorAux.Add( _sensorLuz[ 5 ] ) ;
   SensorAux.Add( _sensorLuz[ 6 ] ) ;
   _corredor[ 3 ].Create( 'H3', SensorAux ) ;
   _sensorLuz[ 5 ].AdcionarCorredor( _corredor[ 3 ] ) ;
   _sensorLuz[ 6 ].AdcionarCorredor( _corredor[ 3 ] ) ;

   inherited Create( ) ;
end ; // TAndar.Create


// mtodo destrutor de TAndar
destructor TAndar.Destroy ;
var
   i : Integer ;
begin

// destri a lista de salas do andar mapeadas pelo sistema de controle de luz
   for i := 0 to _sala.Count - 1 do
   begin
      TSala( TList( _sala ).Items[ i ] ).Destroy( ) ;
   end ; // for

// destri as 3 sees de corredor do andar
   for i := 1 to 3 do
   begin
      _corredor[ i ].Destroy( ) ;
   end ; // for

// destri os 6 sensores de luz externo que mapeiam o andar
   for i := 1 to 6 do
   begin
      _sensorLuz[ i ].Destroy( ) ;
   end ; // for

   //Segue o mtodo de destruio da Superclasse
   inherited Destroy( ) ;

end ; // TAndar.Destroy


// mtodo que fornece o consumo presente e passado de energia
// o presente  o retorno do mtodo
// o passado  retornado na varivel
function TAndar.obterEnergia( var iEnerPass : integer ) : Integer ;
var
   i        ,           // contador auxiliar para percorrer salas e sees
   iEnergia : Integer ; // armazena energia presente
begin

   iEnergia := 0 ;  // inicializa energia presente

// soma energia total consumida pelas salas
   for i := 0 to _sala.Count - 1 do
   begin
      iEnergia := iEnergia + TSala( _sala[ i ] ).obterStatus( )  ;
   end ; // for

// soma energia total consumida pelas sees de corredor
   for i := 1 to 3 do
   begin

   // soma a energia consumida pelo grupo de luzes de corredor somente se ele
   // estiver aceso
      if ( _corredor[ i ].obterLuz( ).obterEstado( ) )
      then
      begin
         iEnergia := iEnergia + I_MAX_CORR ;
      end ; // if

   end ; // for

   iEnerPass := iEnerPass + iEnergia ; // incrementa energia passada
   result := iEnergia ;                // retorna a energia presente

end ; // TAndar.obterEnergia


// simula falha de sinal de sistema de controle ativo
procedure TAndar.simularFalha ;
var
   i : integer ; // contador auxiliar para percorrer todas as salas
begin
   _falha := NOT _falha ; // falha do sinal SSA  um toggle

   if ( _falha = true )                  // acende todas os grupos de luzes
   then                                  // de sala a 100% se o sinal de sistema
   begin                                 // de controle ativo estiver falhado
      for i := 0 to _sala.Count - 1 do
      begin
         TSala( _sala[ i ] ).acenderLuz( ) ;
      end ; // for
   end ; // if

end ; // TAndar.simularFalha


// devolve uma sala mapeada pelo sistema de controle de luz do andar
function TAndar.obterSala( pvsala : String ) : TSala ;
var
   i: integer;
   TempSala: TSala;
   Locade: Boolean;
begin
     i:= 0;
     Locade:= FALSE;

     //Percorre as salas procurando a que deseja-se
     while (i <= _Sala.Count - 1) and not (Locade) do
           begin
           TempSala:= _sala.Items[i];
           //Comparo as salas pelos nomes
           if (TempSala.obterNome = pvSala) then
              Locade:= TRUE
           else
               inc (i);
           end; //while

     if (Locade) then
        result := _sala.Items[ i ]
     else
         result:= Nil;

{   if ( ( sala > -1 )            // testa se ID da sala  vlido (os ID's vo de
    and ( sala < _sala.Count ) ) // zero a Count - 1)
   then
   begin
      result := _sala.Items[ sala ] ; // ID  vlido, retorna a sala desejada
   end // if then
   else
   begin
      result := nil ; // ID  invlido, retorna um valor invlido
   end ; // if else}
end ; // TAndar.obterSala


function TAndar.obterCorredor( pvCorredor : String ) : TCorredor ;
var
   i: integer;
   TempCorr: TCorredor;
   Locade: Boolean;
begin
     i:= 1;
     Locade:= FALSE;

     //Percorre as salas procurando a que deseja-se
     while (i <= 3) and not (Locade) do
           begin
           TempCorr:= _Corredor[i];
           //Comparo as salas pelos nomes
           if (TempCorr.obterNome = pvCorredor) then
              Locade:= TRUE
           else
               inc (i);
           end; //while

     if (Locade) then
        result := _Corredor[i]
     else
         result:= Nil;
         
{   if ( ( corredor > 0 )    // testa se ID da seo de corredor  vlido (os
    and ( corredor < 4 ) )  // ID's vo de 1 a 3)
   then
   begin
      result := _corredor[ corredor ] ; // ID  vlido, retorna a seo desejada
   end // if then
   else
   begin
      result := nil ; // ID  invlido, retorna um valor invlido
   end ; // if else}
end ; // TAndar.obterCorredor


// reinicializa a simulao do sistema de controle de luz
class procedure TAndar.iniciarSimulacao ( var poAndar: TAndar );
begin
   poAndar.Destroy( ) ;
   poAndar := TAndar.Create( ) ;
end ; // TAndar.iniciarSimulacao


// trata incremento da intensidade da iluminao ambiente
procedure TAndar.incIntensidadeAmbiente ;
var
   i : Integer ; // contador auxiliar para percorrer os sensores externos de luz
begin
   Inc( _intensidade ) ; // incrementa intensidade de iluminao ambiente

// no deixa a intensidade ambiente passar o valor mximo permitido
   if ( _intensidade > I_MAX_INTESIDADE )
   then
   begin
      _intensidade := I_MAX_INTESIDADE ;
   end ; // if

   // a intensidade que chega ao sensor  multiplicada pela razo da direo
   // cada sensor recebe uma frao diferente de intensidade, que  igual a i / 6
   // da intensidade multiplicada a razo da direo
   // o mtodo definirEstado( ) garante que a intensidade recebida pelo sensor
   for i := 1 to 6 do
   begin
      _sensorLuz[ i ].definirEstado( _intensidade * _direcao * i div 2160 ) ;
   end ; // for

end ; // TAndar.incIntensidadeAmbiente


// trata decremento da intensidade da iluminao ambiente
procedure TAndar.decIntensidadeAmbiente ;
var
   i : Integer ; // contador auxiliar para percorrer os sensores externos de luz
begin
   Dec( _intensidade ) ; // incrementa intensidade de iluminao ambiente

   if ( _intensidade < 0 ) // no deixa a intensidade ambiente ser negativa
   then
   begin
      _intensidade := 0 ;
   end ; // if

   // a intensidade que chega ao sensor  multiplicada pela razo da direo
   // cada sensor recebe uma frao diferente de intensidade, que  igual a i / 6
   // da intensidade multiplicada a razo da direo
   // o mtodo definirEstado( ) garante que a intensidade recebida pelo sensor
   for i := 1 to 6 do
   begin
      _sensorLuz[ i ].definirEstado( _intensidade * _direcao * i div 2160 ) ;
   end ; // for

end ; // TAndar.decIntensidadeAmbiente


// trata mudana da direo da iluminao ambiente
procedure TAndar.mudarDirecaoAmbiente( direcao : Integer ) ;
var
   i : Integer ; // contador auxiliar para percorrer os sensores externos de luz
begin
   if ( direcao < 1 ) // zero grau ou graus negativos = 360 graus
   then
   begin
      direcao := 360 ;
   end ; // if

   // direo no pode ser maior que 360, se for, ela  sucessivamente diminuda
   // at atingir um valor entre 1 e 360
   while ( direcao > 360 ) do
   begin
      direcao := direcao - 360 ;
   end ; // if

   _direcao := direcao ; // a direo recebe seu novo valor

   // a intensidade que chega ao sensor  multiplicada pela razo da direo
   // cada sensor recebe uma frao diferente de intensidade, que  igual a i / 6
   // da intensidade multiplicada a razo da direo
   // o mtodo definirEstado( ) garante que a intensidade recebida pelo sensor
   for i := 1 to 6 do
   begin
      _sensorLuz[ i ].definirEstado( _intensidade * _direcao * i div 2160 ) ;
   end ; // for

end ; // TAndar.mudarDirecaoAmbiente


// retorna a direo da iluminao ambiente
function TAndar.obterDirecaoAmbiente : Integer ;
begin
   result := _direcao ;
end ; // TAndar.obterDirecaoAmbiente

// retorna a intensidade da iluminao ambiente
function TAndar.obterIntensidadeAmbiente : Integer ;
begin
   result := _intensidade ;
end ; // TAndar.obterIntensidadeAmbiente

// retorna um sensor externo de luz que mapeia o andar
function TAndar.obterSensorLuz( sensorLuz : Integer ) : TSensorLuz ;
begin
   if ( ( sensorLuz > 0 )    // testa se id do sensor de luz  vlido (entre 1
    and ( sensorLuz < 7 ) )  // e 6)
   then
   begin
      result := _sensorLuz[ sensorLuz ] ; // se for, retorna o sensor desejado
   end // if then
   else
   begin
      result := nil ; // seno for, retorna um valor invlido
   end ; // if else
end ; // TAndar.obterSensorLuz


// retorna indcao de falha no sinal de sistema de controle ativo
function TAndar.obterFalha : Boolean ;
begin
   result := _falha ;
end ; // TAndar.obterFalha

end .

// Andares.pas -> v1.4
