unit Corredor;

interface

uses Classes         ,
     LuzesCorredor   ,
     Portas          ,
     Constantes      ,
     SensorMovimento ,
     Timer           ;

Type
    TCorredor = class
      private
        _T2           : Integer                          ; // T2  em minutos
        _ocupacao     : Integer                          ; // ocupao de seo
        _timer        : TTimer                            ; // temporizador
        _luz          : TLuzCorredor                      ; // grupo de luz
        _pushbutton   : array[ 1..3 ] of Boolean         ; // 3 pushbuttons da luz
        _porta        : array[ 1..2 ] of TPorta           ; // 2 portas de extremo
        _sensor       : array[ 1..3 ] of TSensorMovimento ; // 3 sensores de mov
        _sensorLuz    : array[ 1..2 ] of TSensorLuz       ; // 2 sensores de luz
        _nomeCorredor : string                           ; // nome do corredor
      public
        constructor Create( szNome : string ;
                            sensor : TList   )         ;
        destructor  Destroy                            ; override        ;
        function    obterT2                            : Integer         ;
        procedure   definirT2( novoT2 : Integer )      ;
        procedure   incOcupacao                        ;
        procedure   decOcupacao                        ;
        procedure   manipularBotao( botao : Integer )  ;
        function    obterPorta( numero : Integer )     : TPorta           ;
        procedure   apagarLuz                          ;
        procedure   acenderLuz                         ;
        function    obterOcupacao                      : Integer         ;
        function    obterSensorMov( sensor : Integer ) : TSensorMovimento ;
        function    obterLuz                           : TLuzCorredor     ;
        function    obterSensorLuz( sensor : Integer ) : TSensorLuz       ;
        function    obterBotao( botao : Integer      ) : Boolean         ;
        function    obterNome                          : string          ;
        procedure   sinalizarT2                        ;
      end ; // Corredor

implementation

// cria e inicializa a classe Corredor, que modela uma seo de corredor
constructor Corredor.Create( szNome : string ;
                             sensor : TList   ) ;
begin
   _nomeCorredor := szNome ;         // insere nome de seo de corredor
   _sensorLuz[ 1 ] := SensorLuz( sensor.Items[ 0 ] ) ;  // insere sensores
   _sensorLuz[ 2 ] := SensorLuz( sensor.Items[ 1 ] ) ;  // externos de luz
   _T2 := 0 ;                        // T2  inicializado como zero minutos.
   _ocupacao := 0 ;                  // seo de corredor comea vazia
   _timer := Timer.Create( nil, Self ) ; // cria timer para seo de corredor
   _luz  := LuzCorredor.Create( ) ;  // cria o grupo de luz da seo de corredor
   _porta[ 1 ] := Porta.Create( ) ;  // cria uma porta de extremo de seo
   _porta[ 2 ] := Porta.Create( ) ;  // cria a outra porta de extremo de seo
   _sensor[ 1 ] := SensorMovimento.Create( ) ;  // cria os trs sensores de
   _sensor[ 2 ] := SensorMovimento.Create( ) ;  // movimento que mapeiam a seo
   _sensor[ 3 ] := SensorMovimento.Create( ) ;  // de corredor
   _pushbutton[ 1 ] := true ;                // inicializa pushbuttons para que
   _pushbutton[ 2 ] := true ;                // eles permitam acender a luz
   _pushbutton[ 3 ] := true ;                // quando acionados
   inherited Create( ) ;
end ; // Corredor.Create


// mtodo destrutor de Corredor
destructor Corredor.Destroy ;
begin
   _luz.Destroy( ) ;          // destri grupo de luz de seo de corredor
   _porta[ 1 ].Destroy( ) ;   // destri as portas de
   _porta[ 2 ].Destroy( ) ;   // extremo de seo de corredor
   _sensor[ 1 ].Destroy( ) ;  // destri os trs
   _sensor[ 2 ].Destroy( ) ;  // sensores de movimento
   _sensor[ 3 ].Destroy( ) ;  // da seo de corredor
   _timer.Destroy( ) ;        // destri o temporizador da seo de corredor
   inherited Destroy( ) ;
end ; // Corredor.Destroy


// devlove o valor de T2, que  dado em minutos
function  Corredor.obterT2 : Integer ;
begin
   Result := _T2 ;
end ; // Corredor.obterT2


// define um novo valor para T2
procedure Corredor.definirT2( novoT2 : Integer ) ;
begin
   if ( novoT2 < 0 ) // testa se o novo T2  negativo
   then
   begin
      novoT2 := 0 ;  // se for,  considerado como sendo zero
   end ; // if

   _T2 := novoT2 ;   // T2 recebe o novo T2
end ; // Corredor.definirT2


// trata aumento de ocupao de uma seo de corredor
procedure Corredor.incOcupacao ;
var
   iIntens : integer ;  // intensidade captada pelos sensores de luz externos
begin
   Inc( _ocupacao ) ;                       // incrementa ocupao

   if ( ( _sensor[ 1 ].obterEstado( ) = 0 )  // testa se antes no era detectado
    and ( _sensor[ 2 ].obterEstado( ) = 0 )  // movimento
    and ( _sensor[ 3 ].obterEstado( ) = 0 ) )
   then
   begin

   // como agora a seo agora est ocupada, h movimento
      _sensor[ 1 ].definirEstado( 1 ) ;
      _sensor[ 2 ].definirEstado( 1 ) ;
      _sensor[ 3 ].definirEstado( 1 ) ;

   // intensidade que chega no corredor  a mdia aritmtica do valor fornecido
   // pelos 2 sensores externos de luz
      iIntens := ( _sensorLuz[ 1 ].obterEstado( )
              + _sensorLuz[ 2 ].obterEstado( ) ) div 2 ;

      if ( ( iIntens < I_LUZ_SEGURA )      // testa se intensidade  segura
        or ( _sensorLuz[ 1 ].obterFalha( ) = true )   // testa se os sensores de
        or ( _sensorLuz[ 2 ].obterFalha( ) = true ) ) // luz esto falhados

      then
      begin
         acenderLuz( ) ; // se  verdadeiro,  preciso acender a luz da seo
      end ; // if

   end ; // if
end ; // Corredor.incOcupacao


// mtodo que trata diminuio de ocupao de seo de corredor
procedure Corredor.decOcupacao ;
begin

   if ( _ocupacao <> 0 )   // s diminui ocupacao se h gente na sala
   then
   begin
      Dec( _ocupacao ) ;  // decrementa ocupao

      if ( _ocupacao = 0 )
      then
      begin

      // se algum sensor de movimento estiver falhado, ele vai agir como se
      // seo de corredor estivesse ocupada, ou seja, seu estado no ser zero
      // (o mtodo definirEstado s define novo estado se no h falha)
         _sensor[ 1 ].definirEstado( 0 ) ;
         _sensor[ 2 ].definirEstado( 0 ) ;
         _sensor[ 3 ].definirEstado( 0 ) ;

      end ; // if

   end ; // if

// se no  detectado movimento na seo de corredor (se depois de decrementar a
// ocupao no havia mais ningum e no h sensores de movimento falhados), o
// temporizador  ativado
   if ( ( _sensor[ 1 ].ObterEstado( ) = 0 )
    and ( _sensor[ 2 ].ObterEstado( ) = 0 )
    and ( _sensor[ 3 ].ObterEstado( ) = 0 ) )
   then
   begin
      _timer.definirTx( 2, _T2 ) ; // define T2 no temporizador
      _timer.Resume( ) ;           // reinicia o temporizador
   end ; // if

end ; // Corredor.decOcupacao


// mtodo que trata apertar de pushbutton de seo de corredor
procedure Corredor.manipularBotao( botao : Integer ) ;
begin

   if ( ( botao > 0 )    // testa se id de pushbutton  vlido, ou seja, se
    and ( botao < 4 ) )  // est entre 1 e 3

   then
   begin

   // se est, testa se o grupo de luz da seo de corredor est aceso e se os
   // sensores de luz que mapeiam a seo no esto falhados
      if ( ( _luz.obterEstado( ) = true )
       and ( _sensorLuz[ 1 ].obterFalha( ) = false )
       and ( _sensorLuz[ 2 ].obterFalha( ) = false ) )

      then
      begin
      // se estava aceso e os sensores de luz estavam funcionando, o grupo de
      // luz ser apagado e os pushbuttons permitiro acend-lo posteriormente
         _pushbutton[ 1 ] := true ;
         _pushbutton[ 2 ] := true ;
         _pushbutton[ 3 ] := true ;
         _luz.definirEstado( false ) ;
       end // if then

      // seno, o grupo de luz da seo de corredor tem que estar aceso
       else
       begin
          acenderLuz( ) ; // se estava apagada, a luz ser acesa
       end ; // if else

    end ; // if

end ; // Corredor.manipularBotao


// devolve uma porta de extremo de seo de corredor
function  Corredor.obterPorta( numero : Integer) : TPorta ;
begin
   if ( ( numero < 1 )   // s h duas portas, logo, seus IDS's devem ser 1 ou 2
    and ( numero > 2 ) )
   then
   begin
      result := nil ;  // se o ID da porta  invlido, retorna ponteiro invlido
   end // if then
   else
   begin
      result := _porta[ numero ] ;  // seno, retorna a porta desejada
   end ; // if else
end ; // Corredor.obterPorta


// apaga as luzes de seo de corredor se ela estiver sem movimento
// no se deve apagar luz de seo sem ocupao simplesmente pois deve-se
// considerar as possveis falhas nos sensores de movimento
procedure Corredor.apagarLuz ;
begin
   if ( ( _sensor[ 1 ].obterEstado( ) = 0 )   // testa se algum dos sensores de
    and ( _sensor[ 2 ].obterEstado( ) = 0 )   // movimento detectaram presena
    and ( _sensor[ 3 ].obterEstado( ) = 0 ) )
   then                                    // se no detectaram, o Facility
   begin                                   // Manager pode apagar o grupo de luz
      _luz.definirEstado( false ) ;   // apaga o grupo de luz
      _pushbutton[ 1 ] := true ;      // e coloca os pushbuttons como true, ou
      _pushbutton[ 2 ] := true ;      // seja, ao apert-los, a luz da seo de
      _pushbutton[ 3 ] := true ;      // corredor vai acender
   end ; // if
end ; // Corredor.apagarLuz


// acende luz de seo de corredor
procedure Corredor.acenderLuz ;
begin
   _luz.definirEstado( true ) ;  // acende grupo de luz de seo de corredor

   if ( ( _sensorLuz[ 1 ].obterFalha( ) = false )   // testa se os sensores de
    and ( _sensorLuz[ 2 ].obterFalha( ) = false ) ) // luz esto funcionando
   then
   begin
      _pushbutton[ 1 ] := false ;  // pode apagar a luz atravs dos
      _pushbutton[ 2 ] := false ;  // pushbuttons se os sensores de luz
      _pushbutton[ 3 ] := false ;  // estiverem funcionando
   end ; // if then

end ; // Corredor.acenderLuz


// retorna o nmero de pessoas na seo de corredor
function Corredor.obterOcupacao : Integer ;
begin
   result := _ocupacao ;
end ; // Corredor.obterOcupacao


function Corredor.obterSensorMov( sensor : Integer ) : TSensorMovimento ;
begin
   if ( ( sensor < 1 )   // testa se ID de sensor de movimento  vlido, ou
    and ( sensor > 3 ) ) // seja, se est entre 1 e 3
   then
   begin
      result := nil ;    // se for invlido, retorna um valor invlido
   end // if then
   else
   begin
      result := _sensor[ sensor ] ; // seno, retorna o sensor de movimento
   end ; // if else
end ; // Corredor.obterSensorMov


// retorna o nico grupo de luz da seo de corredor
function  Corredor.obterLuz : LuzCorredor ;
begin
   result := _luz ;
end ; // Corredor.obterLuz


// retorna um dado sensor de luz que mapeia a seo de corredor
function Corredor.obterSensorLuz( sensor : Integer ) : TSensorLuz ;
begin
   if ( ( sensor > 0 )    // testa se ID de sensor  um nmero entre 1 e 2 (2 
    and ( sensor < 3 ) )  // o nmero de sensores de luz que mapeiam a seo)
   then
   begin
      result := _SensorLuz[ sensor ] ; // se ID  vlido, retorna sensor de luz
   end // if then
   else
   begin
      result := nil ; // seno, retorna um valor invlido
   end ; // if else
end ; // Corredor.obterSensorLuz


// retorna o estado de um dado pushbutton da seo de corredor; na verdade,
// todos os pushbuttons de uma seo de corredor possuem o mesmo estado
function Corredor.obterBotao( botao : Integer ) : Boolean ;
begin
   if ( ( botao > 0 )   // testa se boto  um nmero entre 1 e 3 (h 3
    and ( botao < 4 ) ) // pushbuttons na seo de corredor)
   then
   begin
      result := _pushbutton[ botao ] ; // se boto  vlido, retorna estado
   end // if then
   else
   begin
      result := false ; // seno, retorna false
   end ; // if else
end ; // Corredor.obterBotao


// retorna o nome da seo de corredor dado no mapa do andar
function Corredor.obterNome ;
begin
   result := _nomeCorredor ;
end ; // Corredor.obterNome


// a sinalizao de T2 implica em apenas apagar o grupo de luz da seo de
// corredor se no houver movimento
procedure Corredor.sinalizarT2 ;
begin
   apagarLuz( ) ;
end ; // Corredor.sinalizarT2

end.
