// v1.4
// Unidade que trata dos grupos de luzes de sala
unit LuzesSala ;


interface

uses Classes   ,
     Constantes;

type
   TLuzSala = class
   private
      _insPadrao    : Integer ; // nvel de iluminao padro para o grupo
      _insEscolhida : Integer ; // nvel de intensidade escolhida para o grupo
      _estado       : Integer ; // nvel de intensidade atual para o grupo
   public
      constructor Create                                                     ;
      destructor  Destroy                                         ; override ;
      function    obterInsPadrao                                  : Integer  ;
      procedure   definirInsPadrao( novaIntensidade : Integer )              ;
      function    obterInsEscolhida                               : Integer  ;
      procedure   definirInsEscolhida( novaIntensidade : Integer )           ;
      function    obterEstado                                     : Integer  ;
      procedure   definirEstado( estado : Integer )                          ;
   end ; // TLuzSala


implementation

// construtor de TLuzSala
// cria e inicializa os atributos de uma luz de sala para serem todos nulos
constructor TLuzSala.Create ;
begin
   _estado := 0 ;       // no estado inicial as luzes esto apagadas
   _insPadrao := 0 ;    // no estado inicial a intensidade padro  nula
   _insEscolhida := 0 ; // no estado inicial a intensidade escolhida  nula
   inherited Create( ) ;
end ; // TLuzSala.Create


// mtodo destrutor para a classe TLuzSala
destructor TLuzSala.Destroy ;
begin
   // no faz nada!
   inherited Destroy( ) ;
end ; // TLuzSala.Destroy


// devolve o nvel de iluminao padro de grupo de luzes de sala
function TLuzSala.obterInsPadrao : Integer ;
begin
   result := _insPadrao ;
end ; // TLuzSala.obterInsPadrao


// define um novo nvel de iluminao padro para grupo de luzes de sala
procedure TLuzSala.definirInsPadrao( novaIntensidade : Integer ) ;
begin

// se a nova intensidade for negativa, ela  considerada como sendo zero
   if ( novaIntensidade < 0 )
   then
   begin
      novaIntensidade := 0 ;
   end ; // if

// se a nova intensidade for maior que o mximo de intensidade permitido para
// grupo de luzes de sala, ela  considerada como sendo o mximo permitido
   if ( novaIntensidade > I_MAX_SALA )
   then
   begin
      novaIntensidade := I_MAX_SALA ;
   end ; // if

// nvel de iluminao padro de grupo de luz de sala recebe nova intensidade
   _insPadrao := novaIntensidade ;

end ; // TLuzSala.definirInsPadrao


// devolve o nvel de iluminao escolhido de grupo de luzes de sala
function TLuzSala.obterInsEscolhida : Integer ;
begin
   result := _insEscolhida ;
end ; // TLuzSala.obterInsEscolhida


// define um novo nvel de iluminao escolhida para grupo de luzes de sala
procedure TLuzSala.definirInsEscolhida( novaIntensidade : Integer ) ;
begin

// se a nova intensidade for negativa, ela  considerada como sendo zero
   if ( novaIntensidade < 0 )
   then
   begin
      novaIntensidade := 0 ;
   end ; // if

// se a nova intensidade for maior que o mximo de intensidade permitido para
// grupo de luzes de sala, ela  considerada como sendo o mximo permitido
   if ( novaIntensidade > I_MAX_SALA )
   then
   begin
      novaIntensidade := I_MAX_SALA ;
   end ; // if

// nvel de iluminao escolhida de grupo de luz de sala recebe nova intensidade
   _insEscolhida := novaIntensidade ;

end ; // TLuzSala.definirInsEscolhida


// devolve o nvel de iluminao atual (o estado) de grupo de luzes de sala
function TLuzSala.obterEstado : Integer ;
begin
   result := _estado ;
end ; // TLuzSala.obterEstado


// define um novo nvel de iluminao atual para grupo de luzes de sala
procedure TLuzSala.definirEstado( estado : Integer ) ;
begin

// se o novo estado for negativo, ele  considerado como sendo zero
   if ( estado < 0 )
   then
   begin
      estado := 0 ;
   end ; // if

// se a novo estado for maior que o mximo de intensidade permitido para
// grupo de luzes de sala, ele  considerade como sendo o mximo permitido
   if ( estado > I_MAX_SALA )
   then
   begin
      estado := I_MAX_SALA ;
   end ; // if

// nvel de iluminao atual de grupo de luz de sala recebe novo estado
   _estado := estado ;

end ; // TLuzSala.definirEstado

end .

// TLuzSala.pas -> v1.4
