// v1.10
// unidade que trata de salas, sees de corredor, sensores e temporizadores
unit Salas ;

interface

uses Classes         ,
     LuzesSala       ,
     Porta           ,
     Constantes      ,
     SensorMovimento ,
     Sensor          ,
     LuzesCorredor   ;

type
    TmyTimer     = Class  ;
    TSensorLuz = Class  ;
    TCorredor  = Class  ;

   //TSala
   TSala = class
   private
      _T1          : Integer                   ; // T1  definido em minutos
      _T3          : Integer                   ; // T3  definido em minutos
      _ocupacao    : Integer                   ; // ocupao de uma sala
      _timer       : TmyTimer                    ; // temporizador
      _luz         : array[ 1..2 ] of TLuzSala ; // conjunto de grupo de luzes
      _pushbutton  : array[ 1..2 ] of Boolean  ; // 2 pushbuttons para cada grupo
      _porta       : TList                     ; // lista de portas da sala
      _sensor      : TSensorMovimento          ; // senor de movimento
      _sensorLuz   : TList                     ; // lista de sensores de luz
      _nomeSala    : string                    ; // Nome da sala no mapa
   public
      constructor Create( szNome     : string ;
                          sensor     : TList  ;
                          iNumPortas : Integer )     ;
      destructor  Destroy                            ; override        ;
      function    obterT1                            : Integer         ;
      procedure   definirT1( novoT1 : Integer )      ;
      function    obterT3                            : Integer         ;
      procedure   definirT3( novoT3 : Integer )      ;
      function    obterOcupacao                      : Integer         ;
      procedure   incOcupacao                        ;
      procedure   decOcupacao                        ;
      function    obterStatus                        : Integer         ;
      procedure   manipularBotao( botao : Integer )  ;
      function    obterPorta( porta : Integer )      : TPorta           ;
      function    obterSensorMov                     : TSensorMovimento ;
      function    obterLuz( luz : Integer )          : TLuzSala         ;
      procedure   apagarLuz                          ;
      procedure   acenderLuz                         ;
      function    obterSensorLuz( sensor : Integer ) : TSensorLuz       ;
      function    obterBotao( botao : Integer )      : Boolean         ;
      function    obterNome                          : string          ;
      procedure   sinalizarT1                        ;
      procedure   sinalizarT3                        ;
   end ; // TSala

   //TmyTimer
   TmyTimer = class ( TThread )
     private
        _T        : array[ 1..3 ] of Integer ;  // array de T's (T1, T2 e T3)
        _sala     : TSala                     ;  // sala mapeada pelo timer
        _corredor : TCorredor                 ;  // corredor mapeado pelo timer
     protected
        constructor Create( NovaSala     : TSala    ;
                            NovoCorredor : TCorredor )  ;
        //destructor  Destroy                            ; override ;
        procedure   Execute                            ; override ;
        procedure   incTx( indice : Integer )          ;
        procedure   decTx( indice : Integer )          ;
        procedure   definirTx( indice : Integer ;
                               valor  : Integer  )     ;
        function    obterTx( indice : Integer )        : Integer ;
     public
        destructor  Destroy                            ; override ;
     end ; // TmyTimer

   TSensorLuz = class ( TSensor )
     private
       _sala     : TList    ;  // lista de salas que ele mapeia
       _corredor : TCorredor ;  // TCorredor que ele mapeia
     public
       constructor Create                                      ;
       destructor  Destroy                                     ; override ;
       procedure   adcionarSala( NovaSala : TSala )             ;
       procedure   adcionarCorredor( NovoCorredor : TCorredor ) ;
       function    obterSala( numero : Integer )               : TSala     ;
       function    obterCorredor                               : TCorredor ;
       procedure   simularFalha                                ;
     end ; // TSensorLuz
    TCorredor = class
      private
        _T2           : Integer                          ; // T2  em minutos
        _ocupacao     : Integer                          ; // ocupao de seo
        _timer        : TmyTimer                            ; // temporizador
        _luz          : TLuzCorredor                      ; // grupo de luz
        _pushbutton   : array[ 1..3 ] of Boolean         ; // 3 pushbuttons da luz
        _porta        : array[ 1..2 ] of TPorta           ; // 2 portas de extremo
        _sensor       : array[ 1..3 ] of TSensorMovimento ; // 3 sensores de mov
        _sensorLuz    : array[ 1..2 ] of TSensorLuz       ; // 2 sensores de luz
        _nomeCorredor : string                           ; // nome do TCorredor
      public
        constructor Create( szNome : string ;
                            sensor : TList   )         ;
        destructor  Destroy                            ; override        ;
        function    obterT2                            : Integer         ;
        procedure   definirT2( novoT2 : Integer )      ;
        procedure   incOcupacao                        ;
        procedure   decOcupacao                        ;
        procedure   manipularBotao( botao : Integer )  ;
        function    obterPorta( numero : Integer )     : TPorta           ;
        procedure   apagarLuz                          ;
        procedure   acenderLuz                         ;
        function    obterOcupacao                      : Integer         ;
        function    obterSensorMov( sensor : Integer ) : TSensorMovimento ;
        function    obterLuz                           : TLuzCorredor     ;
        function    obterSensorLuz( sensor : Integer ) : TSensorLuz       ;
        function    obterBotao( botao : Integer      ) : Boolean         ;
        function    obterNome                          : string          ;
        procedure   sinalizarT2                        ;
      end ; // TCorredor

implementation

uses Andares ;

// Identificador da funcao do Windows que usaremos para dormir no Timer
procedure Sleep( milli : Longint ) ; stdcall ; external 'kernel32.dll' ;

// cria e inicializa a classe TCorredor, que modela uma seo de TCorredor
constructor TCorredor.Create( szNome : string ;
                             sensor : TList   ) ;
begin
   _nomeCorredor := szNome ;         // insere nome de seo de TCorredor
   _sensorLuz[ 1 ] := TSensorLuz( sensor.Items[ 0 ] ) ;  // insere sensores
   _sensorLuz[ 2 ] := TSensorLuz( sensor.Items[ 1 ] ) ;  // externos de luz
   _T2 := 0 ;                        // T2  inicializado como zero minutos.
   _ocupacao := 0 ;                  // seo de TCorredor comea vazia
   _timer := TmyTimer.Create( nil, Self ) ; // cria timer para seo de TCorredor
   _luz  := TLuzCorredor.Create( ) ;  // cria o grupo de luz da seo de TCorredor
   _porta[ 1 ] := TPorta.Create( ) ;  // cria uma porta de extremo de seo
   _porta[ 2 ] := TPorta.Create( ) ;  // cria a outra porta de extremo de seo
   _sensor[ 1 ] := TSensorMovimento.Create( ) ;  // cria os trs sensores de
   _sensor[ 2 ] := TSensorMovimento.Create( ) ;  // movimento que mapeiam a seo
   _sensor[ 3 ] := TSensorMovimento.Create( ) ;  // de TCorredor
   _pushbutton[ 1 ] := true ;                // inicializa pushbuttons para que
   _pushbutton[ 2 ] := true ;                // eles permitam acender a luz
   _pushbutton[ 3 ] := true ;                // quando acionados
   inherited Create( ) ;
end ; // TCorredor.Create


// mtodo destrutor de TCorredor
destructor TCorredor.Destroy ;
begin
   _luz.Destroy( ) ;          // destri grupo de luz de seo de TCorredor
   _porta[ 1 ].Destroy( ) ;   // destri as portas de
   _porta[ 2 ].Destroy( ) ;   // extremo de seo de TCorredor
   _sensor[ 1 ].Destroy( ) ;  // destri os trs
   _sensor[ 2 ].Destroy( ) ;  // sensores de movimento
   _sensor[ 3 ].Destroy( ) ;  // da seo de TCorredor
   _timer.Destroy( ) ;        // destri o temporizador da seo de TCorredor
   inherited Destroy( ) ;
end ; // TCorredor.Destroy


// devlove o valor de T2, que  dado em minutos
function  TCorredor.obterT2 : Integer ;
begin
   Result := _T2 ;
end ; // TCorredor.obterT2


// define um novo valor para T2
procedure TCorredor.definirT2( novoT2 : Integer ) ;
begin
   if ( novoT2 < 0 ) // testa se o novo T2  negativo
   then
   begin
      novoT2 := 0 ;  // se for,  considerado como sendo zero
   end ; // if

   _T2 := novoT2 ;   // T2 recebe o novo T2
end ; // TCorredor.definirT2


// trata aumento de ocupao de uma seo de TCorredor
procedure TCorredor.incOcupacao ;
var
   iIntens : integer ;  // intensidade captada pelos sensores de luz externos
begin
   Inc( _ocupacao ) ;                       // incrementa ocupao

   if ( ( _sensor[ 1 ].obterEstado( ) = 0 )  // testa se antes no era detectado
    and ( _sensor[ 2 ].obterEstado( ) = 0 )  // movimento
    and ( _sensor[ 3 ].obterEstado( ) = 0 ) )
   then
   begin

   // como agora a seo agora est ocupada, h movimento
      _sensor[ 1 ].definirEstado( 1 ) ;
      _sensor[ 2 ].definirEstado( 1 ) ;
      _sensor[ 3 ].definirEstado( 1 ) ;

   // intensidade que chega no TCorredor  a mdia aritmtica do valor fornecido
   // pelos 2 sensores externos de luz
      iIntens := ( _sensorLuz[ 1 ].obterEstado( )
              + _sensorLuz[ 2 ].obterEstado( ) ) div 2 ;

      if ( ( iIntens < I_LUZ_SEGURA )      // testa se intensidade  segura
        or ( _sensorLuz[ 1 ].obterFalha( ) = true )   // testa se os sensores de
        or ( _sensorLuz[ 2 ].obterFalha( ) = true ) ) // luz esto falhados

      then
      begin
         acenderLuz( ) ; // se  verdadeiro,  preciso acender a luz da seo
      end ; // if

   end ; // if
end ; // TCorredor.incOcupacao


// mtodo que trata diminuio de ocupao de seo de TCorredor
procedure TCorredor.decOcupacao ;
begin

   if ( _ocupacao <> 0 )   // s diminui ocupacao se h gente na sala
   then
   begin
      Dec( _ocupacao ) ;  // decrementa ocupao

      if ( _ocupacao = 0 )
      then
      begin

      // se algum sensor de movimento estiver falhado, ele vai agir como se
      // seo de TCorredor estivesse ocupada, ou seja, seu estado no ser zero
      // (o mtodo definirEstado s define novo estado se no h falha)
         _sensor[ 1 ].definirEstado( 0 ) ;
         _sensor[ 2 ].definirEstado( 0 ) ;
         _sensor[ 3 ].definirEstado( 0 ) ;

      end ; // if

   end ; // if

// se no  detectado movimento na seo de TCorredor (se depois de decrementar a
// ocupao no havia mais ningum e no h sensores de movimento falhados), o
// temporizador  ativado
   if ( ( _sensor[ 1 ].ObterEstado( ) = 0 )
    and ( _sensor[ 2 ].ObterEstado( ) = 0 )
    and ( _sensor[ 3 ].ObterEstado( ) = 0 ) )
   then
   begin
      _timer.definirTx( 2, _T2 ) ; // define T2 no temporizador
      _timer.Resume( ) ;           // reinicia o temporizador
   end ; // if

end ; // TCorredor.decOcupacao


// mtodo que trata apertar de pushbutton de seo de TCorredor
procedure TCorredor.manipularBotao( botao : Integer ) ;
begin

   if ( ( botao > 0 )    // testa se id de pushbutton  vlido, ou seja, se
    and ( botao < 4 ) )  // est entre 1 e 3

   then
   begin

   // se est, testa se o grupo de luz da seo de TCorredor est aceso e se os
   // sensores de luz que mapeiam a seo no esto falhados
      if ( ( _luz.obterEstado( ) = true )
       and ( _sensorLuz[ 1 ].obterFalha( ) = false )
       and ( _sensorLuz[ 2 ].obterFalha( ) = false ) )

      then
      begin
      // se estava aceso e os sensores de luz estavam funcionando, o grupo de
      // luz ser apagado e os pushbuttons permitiro acend-lo posteriormente
         _pushbutton[ 1 ] := true ;
         _pushbutton[ 2 ] := true ;
         _pushbutton[ 3 ] := true ;
         _luz.definirEstado( false ) ;
       end // if then

      // seno, o grupo de luz da seo de TCorredor tem que estar aceso
       else
       begin
          acenderLuz( ) ; // se estava apagada, a luz ser acesa
       end ; // if else

    end ; // if

end ; // TCorredor.manipularBotao


// devolve uma porta de extremo de seo de TCorredor
function  TCorredor.obterPorta( numero : Integer) : TPorta ;
begin
   if ( ( numero < 1 )   // s h duas portas, logo, seus IDS's devem ser 1 ou 2
    and ( numero > 2 ) )
   then
   begin
      result := nil ;  // se o ID da porta  invlido, retorna ponteiro invlido
   end // if then
   else
   begin
      result := _porta[ numero ] ;  // seno, retorna a porta desejada
   end ; // if else
end ; // TCorredor.obterPorta


// apaga as luzes de seo de TCorredor se ela estiver sem movimento
// no se deve apagar luz de seo sem ocupao simplesmente pois deve-se
// considerar as possveis falhas nos sensores de movimento
procedure TCorredor.apagarLuz ;
begin
   if ( ( _sensor[ 1 ].obterEstado( ) = 0 )   // testa se algum dos sensores de
    and ( _sensor[ 2 ].obterEstado( ) = 0 )   // movimento detectaram presena
    and ( _sensor[ 3 ].obterEstado( ) = 0 ) )
   then                                    // se no detectaram, o Facility
   begin                                   // Manager pode apagar o grupo de luz
      _luz.definirEstado( false ) ;   // apaga o grupo de luz
      _pushbutton[ 1 ] := true ;      // e coloca os pushbuttons como true, ou
      _pushbutton[ 2 ] := true ;      // seja, ao apert-los, a luz da seo de
      _pushbutton[ 3 ] := true ;      // TCorredor vai acender
   end ; // if
end ; // TCorredor.apagarLuz


// acende luz de seo de TCorredor
procedure TCorredor.acenderLuz ;
begin
   _luz.definirEstado( true ) ;  // acende grupo de luz de seo de TCorredor

   if ( ( _sensorLuz[ 1 ].obterFalha( ) = false )   // testa se os sensores de
    and ( _sensorLuz[ 2 ].obterFalha( ) = false ) ) // luz esto funcionando
   then
   begin
      _pushbutton[ 1 ] := false ;  // pode apagar a luz atravs dos
      _pushbutton[ 2 ] := false ;  // pushbuttons se os sensores de luz
      _pushbutton[ 3 ] := false ;  // estiverem funcionando
   end ; // if then

end ; // TCorredor.acenderLuz


// retorna o nmero de pessoas na seo de TCorredor
function TCorredor.obterOcupacao : Integer ;
begin
   result := _ocupacao ;
end ; // TCorredor.obterOcupacao


function TCorredor.obterSensorMov( sensor : Integer ) : TSensorMovimento ;
begin
   if ( ( sensor < 1 )   // testa se ID de sensor de movimento  vlido, ou
    and ( sensor > 3 ) ) // seja, se est entre 1 e 3
   then
   begin
      result := nil ;    // se for invlido, retorna um valor invlido
   end // if then
   else
   begin
      result := _sensor[ sensor ] ; // seno, retorna o sensor de movimento
   end ; // if else
end ; // TCorredor.obterSensorMov


// retorna o nico grupo de luz da seo de TCorredor
function  TCorredor.obterLuz : TLuzCorredor ;
begin
   result := _luz ;
end ; // TCorredor.obterLuz


// retorna um dado sensor de luz que mapeia a seo de TCorredor
function TCorredor.obterSensorLuz( sensor : Integer ) : TSensorLuz ;
begin
   if ( ( sensor > 0 )    // testa se ID de sensor  um nmero entre 1 e 2 (2 
    and ( sensor < 3 ) )  // o nmero de sensores de luz que mapeiam a seo)
   then
   begin
      result := _SensorLuz[ sensor ] ; // se ID  vlido, retorna sensor de luz
   end // if then
   else
   begin
      result := nil ; // seno, retorna um valor invlido
   end ; // if else
end ; // TCorredor.obterSensorLuz


// retorna o estado de um dado pushbutton da seo de TCorredor; na verdade,
// todos os pushbuttons de uma seo de TCorredor possuem o mesmo estado
function TCorredor.obterBotao( botao : Integer ) : Boolean ;
begin
   if ( ( botao > 0 )   // testa se boto  um nmero entre 1 e 3 (h 3
    and ( botao < 4 ) ) // pushbuttons na seo de TCorredor)
   then
   begin
      result := _pushbutton[ botao ] ; // se boto  vlido, retorna estado
   end // if then
   else
   begin
      result := false ; // seno, retorna false
   end ; // if else
end ; // TCorredor.obterBotao


// retorna o nome da seo de TCorredor dado no mapa do andar
function TCorredor.obterNome ;
begin
   result := _nomeCorredor ;
end ; // TCorredor.obterNome


// a sinalizao de T2 implica em apenas apagar o grupo de luz da seo de
// TCorredor se no houver movimento
procedure TCorredor.sinalizarT2 ;
begin
   apagarLuz( ) ;
end ; // TCorredor.sinalizarT2

// cria e inicializa sensor de luz
constructor TSensorLuz.Create ;
begin
   self._sala := TList.Create( ) ;  // inicializa lista de sala
   self._corredor := nil ;          // inicializa TCorredor
   inherited Create( ) ;
end ; // SensorLuz.Create


// mtodo destrutor do sensor de luz
destructor TSensorLuz.Destroy ;
begin
   // no faz nada!!
   inherited Destroy( ) ;
end ; // SensorLuz.Destroy


// mtodo para colocar sensor de luz mapeando sala
procedure TSensorLuz.adcionarSala( NovaSala : TSala ) ;
begin
   _sala.Add( NovaSala ) ; // coloca mais uma sala na lista
end ; // SensorLuz.adcionarSala

// mtodo para colocar um sensor de luz mapeando seo de TCorredor
procedure TSensorLuz.adcionarCorredor( NovoCorredor : TCorredor ) ;
begin
   _corredor := NovoCorredor ; // sensor passa a mapear esta seo
end ; // SensorLuz.adcionarCorredor


// devolve objeto da classe Sala dado o seu ID que  mapeado por objeto da
// classe SensorLuz
function TSensorLuz.obterSala( numero : Integer ) : TSala ;
begin
// testa se nmero  vlido
   if ( ( numero > -1 )
    and ( numero < _sala.Count ) )
   then
   begin
      Result := _sala.Items[ numero ] ; // se  vlido, retorna a sala
   end
   else
   begin
       Result := nil ; // seno, retorna ponteiro invlido
   end ;
end ; // SensorLuz.obterSala

// devolve objeto da classe TCorredor que  mapeado por objeto da classe
// SensorLuz
function TSensorLuz.obterCorredor : TCorredor ;
begin
   Result := _corredor ;
end ; // SensorLuz.obterCorredor


// permite falhar ou consertar um sensor externo de luz
// se est funcionando corretamente (_falha = false) vai ficar falhado
// se est falhado (_falha = true) vai passar a funcionar corretamente
procedure TSensorLuz.simularFalha ;
var
   i : Integer ;  // contador auxiliar para setar os grupos de luzes
begin
   _falha := NOT _falha ;  // inverte o indicador de falha

// s trata falha se sensor externo de luz extiver falhado
   if ( _falha = true )
   then
   begin
   // se h movimento na seo de TCorredor, seu grupo de luz deve estar aceso
      if ( ( _corredor.obterSensorMov( 1 ).obterEstado = 1 )
        or ( _corredor.obterSensorMov( 2 ).obterEstado = 1 )
        or ( _corredor.obterSensorMov( 3 ).obterEstado = 1 ) )
      then
      begin
         _corredor.acenderLuz( ) ; // acende grupo de luz de seo de TCorredor
      end ; // if

   // o nvel de iluminao padro em cada grupo de luz de sala deve ser de 100%
      for i := 0 to _sala.Count - 1 do
      begin
         TSala( _sala.Items[ i ] ).obterLuz( 1 ).definirInsPadrao( I_MAX_SALA ) ;
         TSala( _sala.Items[ i ] ).obterLuz( 2 ).definirInsPadrao( I_MAX_SALA ) ;
      end ; // for
   end ; // if

end ; // SensorLuz.simularFalha

// cria e inicializa Timer
constructor TmyTimer.Create( NovaSala     : TSala    ;
                           NovoCorredor : TCorredor ) ;
begin
   _sala := NovaSala ;          // recebe sala a ser mapeada
   _corredor := NovoCorredor ;  // recebe TCorredor a ser mapeado
   _T[ 1 ] := 0 ;               // T1 inicialmente  zero minutos
   _T[ 2 ] := 0 ;               // T2 inicialmente  zero minutos
   _T[ 3 ] := 0 ;               // T3 inicialmente  zero minutos
   inherited Create( true ) ;   // aciona a execuo do Timer
end ; // TmyTimer.Create


// mtodo destrutor de Timer
destructor TmyTimer.Destroy ;
begin
   Terminate( ) ;         // serve para terminar o loop de execute do Timer
   inherited Destroy( ) ;
end ; // TmyTimer.Destroy


// executa o Timer (comea a temporizao)
procedure TmyTimer.Execute ;
var
   i : Integer ; // contador auxiliar
begin
// continua enquanto o temporizador no for destruido ou parado
   while( not Terminated ) do
   begin

      Sleep( TimeUnit ) ; // Aguarda por uma unidade de tempo

      for i := 1 to 3 do // decrementa T1, T2 e T3
      begin
         Dec( _T [ i ] ) ;   // decrementa 1 minuto
      end ; // for

   // s sinaliza T1 se T1 chegou a zero e se o timer  de sala
      if ( ( _T[ 1 ] = 0 )
       and ( _sala <> nil ) )
      then
      begin
         Synchronize( _sala.sinalizarT1 ) ; // sinaliza T1 na sala
      end ; // if

   // s sinaliza T2 se T2 chegou a zero e se o timer  de seo de TCorredor
      if ( ( _T[ 2 ] = 0 )
       and ( _corredor <> nil ) )
      then
      begin
         Synchronize( _corredor.sinalizarT2 ) ; // sinaliza T2 na seo
         Terminate( ) ;                         // termina o loop
      end ; // if

   // s sinaliza T3 se T3 chegou a zero e se o timer  de sala
      if ( ( _T[ 3 ] = 0 )
       and ( _sala <> nil ) )
      then
      begin
         Synchronize( _sala.sinalizarT3 ) ; // sinaliza T3 na sala
         Terminate( ) ;                     // termina o loop
      end ;

   // coloca em zero T1, T2 ou T3 se eles forem menores que zero
      for i := 1 to 3 do
      begin
         if ( _T[ i ] < 0 )
         then
         begin
            _T[ i ] := 0 ;
         end ; // if
      end ; // for

   end ; // while

end ; // TmyTimer.Execute


// incrementa em um minuto um dado Ti
procedure TmyTimer.incTx( indice : Integer ) ;
begin

// testa se indice est entre 1 e 3
   if ( ( indice > 0 )
    and ( indice < 4 ) )
   then
   begin
      Inc( _T[ indice ] ) ;  // incrementa Ti
   end ; // if

end ; // TmyTimer.incTx


// decrementa em um minuto um dado Ti
procedure TmyTimer.decTx( indice : Integer ) ;
begin

// testa se indice est entre 1 e 3
   if ( ( indice > 0 )
    and ( indice < 4 ) )
   then
   begin
      Dec( _T[ indice ] ) ;  // decrementa Ti
   end ; // if

end ; // TmyTimer.decTx


// define um Ti com um dado valor
procedure TmyTimer.definirTx( indice : Integer ;
                            valor  : Integer  ) ;
begin

// testa se indice est entre 1 e 3
   if ( ( indice > 0 )
    and ( indice < 4 ) )
   then
   begin

      if ( valor > 0 ) // testa se novo valor de Ti  positivo
      then
      begin
         _T[ indice ] := valor ; // se novo Ti > 0, Ti recebe novo Ti
      end // if then
      else
      begin
         _T[ indice ] := 0 ; // seno, Ti recebe zero
      end ; // if else

   end ; // if

end ; // TmyTimer.definirTx

// devolve um Ti
function TmyTimer.obterTx( indice : Integer ) : Integer ;
begin

// testa se indice est entre 1 e 3
   if ( ( indice > 0 )
    and ( indice < 4 ) )
   then
   begin
      result := _T[ indice ] ; // devolve o Ti se i  vlido
   end // if then
   else
   begin
      result := -1; // Erro se i  invlido
   end ; // if else

end ; // TmyTimer.obterTx

// cria e inicializa uma sala
constructor TSala.Create( szNome     : string ;
                         sensor     : TList  ;
                         iNumPortas : Integer ) ;
var
   i : Integer ; // contador para auxiliar a criao de portas
begin
   _nomeSala := szNome ;   // insere nome da sala
   _sensorLuz := sensor ;  // insere lista de sensores externos de luz
   _T1 := 0 ;              // T1  inicializado como zero minutos.
   _T3 := 0 ;              // T3  inicializado como zero minutos
   _ocupacao := 0 ;        // sala comea vazia
   _timer := TmyTimer.Create( Self, nil ) ;  // cria temporizador para sala
   _luz[ 1 ] := TLuzSala.Create( ) ;       // cria os dois grupos
   _luz[ 2 ] := TLuzSala.Create( ) ;       // de luzes de sala
   _sensor := TSensorMovimento.Create( ) ; // cria sensor de movimento da sala
   _pushbutton[ 1 ] := true ;   // inicializa pushbuttons para que eles permitam
   _pushbutton[ 2 ] := true ;   // acender a luz quando acionados
   _porta := TList.Create( ) ;  // inicializa portas

// cria todas as portas de sala
   for i := 1 to iNumPortas do     // so criadas iNumPortas para a sala
   begin
      _porta.Add( TPorta.Create( ) ) ; // cada porta  adcionada a lista
   end ; // for

   inherited Create( ) ;
end ; // TSala.Create


// mtodo destrutor para a sala
destructor TSala.Destroy ;
var
   i : Integer ;  // contador auxiliar para destruir as portas
begin
   _luz[ 1 ].Destroy( ) ;  // destri o primeiro grupo de luz
   _luz[ 2 ].Destroy( ) ;  // destri o segundo grupo de luz
   _timer.Destroy( ) ;     // destri o temporizador da sala
   _sensor.Destroy( ) ;    // destri o sensor de movimento da sala

// destri a lista de portas
   for i := 0 to _porta.Count - 1 do
   begin
      TPorta( _porta.Items[ i ] ).Destroy( ) ;
   end ; // for

   inherited Destroy( ) ;
end ; // TSala.Destroy


// retorna o T1 da sala, que  dado em minutos
function TSala.obterT1 : Integer ;
begin
   result := _T1 ;
end ; // TSala.obterT1


// define um novo T1 para a sala, dado em minutos
procedure TSala.definirT1( novoT1 : Integer ) ;
begin
   if ( novoT1 < 0 ) // testa se o novo T1  negativo
   then
   begin
      novoT1 := 0 ;  // se for,  considerado como sendo zero
   end ; // if

   _T1 := novoT1 ;  // atribui novo T1 ao T1 da sala

end ; // TSala.definirT1


// retorna o T3 da sala, que  dado em minutos
function TSala.obterT3 : Integer ;
begin
   result := _T3 ;
end ; // TSala.obterT3


// define um novo T3 para a sala, dado em minutos
procedure TSala.definirT3( novoT3 : Integer ) ;
begin
   if ( novoT3 < 0 ) // testa se o novo T3  negativo
   then
   begin
      novoT3 := 0 ;  // se for,  considerado como sendo zero
   end ; // if

   _T3 := novoT3 ;  // atribui novo T3 ao T3 da sala
end ; // TSala.definirT3


// devolve o nmero de pessoas dentro da sala
function TSala.obterOcupacao : Integer ;
begin
   result := _ocupacao ;
end ; // TSala.obterOcupacao


// mtodo que trata do incremento do nmero de pessoas na sala
procedure TSala.incOcupacao ;
var
   iIntensidade : Integer ; // armazena a intensidade que chega na sala
   i            : Integer ; // contador auxiliar para o clculo da intensidade
begin

   Inc( _ocupacao ) ;      // incrementa ocupao

   if ( _sensor.obterEstado = 0 )  // testa se antes no era detectado movimento

   then
   begin

   // no havia movimento, sala estava vazia e sensor de movimento funcionando
   // corretamente
      _sensor.definirEstado( 1 ) ; // agora haver movimento detectado

      if ( _timer.obterTx( 3 ) = 0 ) // testa se j passou T3
      then
      begin

      // j passou T3

      // pushbuttons acendero a luz a 100% se manipulados
         _pushbutton[ 1 ] := true ;
         _pushbutton[ 2 ] := true ;

         iIntensidade := 0 ; // inicializa intensidade

      // soma as intensidades da iluminao ambiente que cada sensor de luz
      // externo de luz que mapeia esta sala recebe
         for i := 0 to _sensorLuz.Count - 1 do
         begin
            iIntensidade := iIntensidade
                         + TSensorLuz( _sensorLuz.Items[ i ] ).obterEstado( ) ;
         end ; // for

      // calcula a mdia aritmtica das intensidades recebidas pelos sensores
         iIntensidade := iIntensidade div _sensorLuz.Count ;

      // a luz que  acesa na sala  tal que a iluminao ser segura
         _luz[ 1 ].definirEstado( I_LUZ_SEGURA - iIntensidade ) ;
         _luz[ 2 ].definirEstado( I_LUZ_SEGURA - iIntensidade ) ;

      // a intensidade escolhida  a mesma que a atual da sala
         _luz[ 1 ].definirInsEscolhida( I_LUZ_SEGURA - iIntensidade ) ;
         _luz[ 2 ].definirInsEscolhida( I_LUZ_SEGURA - iIntensidade ) ;

      // no h perigo de luminosidade emitida por um grupo de luz ser negativa
      // pois a funo que define as intensidades garante que no h
      // luminosidades negativas

      end // if then

      else
      begin

      // no passou T3

         if ( _timer.obterTx( 1 ) = 0 ) // testa se j passou T1
         then
         begin

         // se passou T1, o nvel de iluminao padro  estabelecido nos dois
         // grupos de luzes
            _luz[ 1 ].definirEstado( _luz[ 1 ].ObterInsPadrao( ) ) ;
            _luz[ 2 ].definirEstado( _luz[ 2 ].ObterInsPadrao( ) ) ;

         end // if then

         else
         begin

         // se no passou T1, o nvel de iluminao escolhido  reestabelecido
         // nos dois grupos de luzes
            _luz[ 1 ].definirEstado( _luz[ 1 ].ObterInsEscolhida( ) ) ;
            _luz[ 2 ].definirEstado( _luz[ 2 ].ObterInsEscolhida( ) ) ;

         end ; // if else

      // acerta os estados dos 2 push buttons
         for i := 1 to 2 do
         begin
            if ( _luz[ i ].ObterEstado = I_MAX_SALA ) // testa se a luz  max
            then
            begin
               _pushbutton[ i ] := false ; // se for, pushbutton poder apagar
            end // if then
            else
            begin
               _pushbutton[ i ] := true ; // seno, poder acender
            end ; // if else
         end ; // for

      end ; // if else

   end ; // if

end ; // TSala.incOcupacao


// mtodo que trata do decremento do nmero de pessoas na sala
procedure TSala.decOcupacao ;
begin

   if ( _ocupacao <> 0 )   // s diminui ocupacao se h gente na sala
   then
   begin
      Dec( _ocupacao ) ;  // decrementa ocupao
      if ( _ocupacao = 0 )
      then
      begin

      // sala fica como se ainda estivesse ocupada se sensor de movimento
      // estiver falhado, pois continuar detectando movimento (definirEstado
      // garante isso)
         _sensor.definirEstado( 0 ) ;

      end ; // if
   end ; // if

// se no  detectado movimento na sala (se depois de decrementar a ocupao no
// havia mais ningum e o sensor de movimento no est falhado), o temporizador
//  ativado
   if ( _sensor.ObterEstado = 0 )
   then
   begin
      _timer.definirTx( 1, _T1 ) ; // define T1 no temporizador
      _timer.definirTx( 3, _T3 ) ; // define T3 no temporizador
      _timer.Resume( ) ;           // reinicia o temporizador
   end ; // if

end ; // TSala.decOcupacao


// devolve energia consumida (iluminao) pela sala
function TSala.obterStatus : Integer ;
begin

// a iluminao consumida  a soma do consumo dos dois grupos de luzes da sala
   Result := _luz[ 1 ].obterEstado( ) + _luz[ 2 ].obterEstado( ) ;

end ; // TSala.obterStatus


// trata manipulao de push button de sala
procedure TSala.manipularBotao( botao : Integer ) ;
begin

   if ( ( botao > 0 )   // testa se botao  vlido (ID entre 1 e 2 )
    and ( botao < 3 ) )
   then
   begin

   // se boto  vlido, testa se pushbutton est no estado para apagar a luz
      if ( _pushbutton[ botao ] = false )
      then
      begin

      // se estava (grupo de luz 100% aceso)
         _pushbutton[ botao ] := true ;    // o boto agora permitir acender
         _luz[ botao ].definirEstado( 0 ) ;       // o grupo de luz ser apagado
         _luz[ botao ].definirInsEscolhida( 0 ) ; // a luz escolhida  de 0 Lux

      end // if then

      else
      begin

      // se no estava 100% acesa
         _pushbutton[ botao ] := false ; // o boto agora permitir apagar

      // o grupo de luz ser aceso a 100%
         _luz[ botao ].definirEstado( I_MAX_SALA ) ;

      // o nvel de iluminao escolhida ser o mximo permitido ao grupo de luz
         _luz[ botao ].definirInsEscolhida( I_MAX_SALA ) ;

      end ; // if else

   end ; // if

end ; // TSala.manipularBotao


// devolve uma porta de sala
function TSala.obterPorta( porta : Integer ) : TPorta ;
begin
   if ( ( porta > -1 )                  // testa se o ID da porta  vlido
    and ( porta < _porta.Count ) )
   then
   begin
      result := _porta.Items[ porta ] ; // se for, retorna a porta desejada
   end // if then
   else
   begin
      Result := nil ; // seno, retorna um valor invlido
   end ; // if else
end ; // TSala.obterPorta


// retorna o nico sensor de movimento que mapeia a sala
function  TSala.obterSensorMov : TSensorMovimento ;
begin
   result := _sensor ;
end ; // TSala.obterSensorMov


// devolve um dado grupo luz de sala
function TSala.obterLuz( luz : Integer ) : TLuzSala ;
begin
   if ( ( luz > 0 )    // testa se ID da luz  vlido, ou seja, se est entre
    and ( luz < 3 ) )  // 1 e 2
   then
   begin
      result := _luz[ luz ] ; // se  vlido, retorna o grupo de luz desejado
   end // if then
   else
   begin
      result := nil ; // seno, retorna um valor invlido
   end ; // if else
end ; // TSala.obterLuz


// apaga as luzes da sala se ela estiver sem movimento; no se deve apagar luz
// da sala sem ocupao simplesmente pois deve-se considerar a possvel falha
// no sensor de movimento
procedure TSala.apagarLuz ;
begin
   if ( _sensor.obterEstado( ) = 0 ) // s apaga as luzes se no h movimento
   then
   begin
      _luz[ 1 ].definirEstado( 0 ) ;  // apaga os 2 grupos de luzes da sala,
      _luz[ 2 ].definirEstado( 0 ) ;  // definindo seus estados como zero
      _pushbutton[ 1 ] := true ;  // e coloca os pushbuttons como true, ou seja,
      _pushbutton[ 2 ] := true ;  // ao apert-los, a luz do grupo vai acender
   end ; // if
end ; // TSala.apagarLuz


// este mtodo  para a falha do sinal de sistema de controle ativo
procedure TSala.acenderLuz ;
begin
   _luz[ 1 ].definirEstado( I_MAX_SALA ) ; // acende ao mximo os grupos de
   _luz[ 2 ].definirEstado( I_MAX_SALA ) ; // luzes da sala
   _pushbutton[ 1 ] := true ;          // apertar os pushbuttons no far com
   _pushbutton[ 2 ] := true ;          // que os grupos de luzes sejam apagados
end ; // TSala.acenderLuz ;


// retorna um dado sensor de luz que mapeia a sala
function TSala.obterSensorLuz( sensor : Integer ) : TSensorLuz ;
begin
   if ( ( sensor > -1 )                 // testa se ID so sensor de luz
    and ( sensor < _sensorLuz.Count ) ) // externo  vlido
   then
   begin
      result := _SensorLuz[ sensor ] ; // se , retorna o sensor de luz dejesado
   end // if then
   else
   begin
      result := nil ; // seno, retorna um valor invlido
   end ; // if else
end ; // TSala.obterSensorLuz


// retorna o estado de um dado pushbutton da sala
function TSala.obterBotao( botao : Integer ) : Boolean ;
begin
   if ( ( botao > 0 )   // testa se boto  um nmero entre 1 e 2 (h 2
    and ( botao < 3 ) ) // pushbuttons na sala)
   then
   begin
      result := _pushbutton[ botao ] ; // se boto  vlido, retorna estado
   end // if then
   else
   begin
      result := false ; // seno, retorna false
   end ; // if else
end ; // TSala.obterBotao


// retorna o nome da sala dado no mapa do andar
function TSala.obterNome ;
begin
   Result := _nomeSala ;
end ; // TSala.obterNome


// sinalizar T1 implica em setar T1 do Timer como zero
procedure TSala.sinalizarT1 ;
begin
   _timer.definirTx( 1, 0 ) ;
end ; // TSala.sinalizarT1


// a sinalizao de T3 implica em apenas apagar o grupo de luz da sala se no
// houver movimento
procedure TSala.sinalizarT3 ;
begin
   apagarLuz( ) ;
end ; // TSala.sinalizarT3

end . // Salas.pas -> v1.10
