unit SensorLuz;

interface

uses Classes       ,
     Salas         ,
     Corredor      ,
     Constantes    ,
     Sensor        ;
Type
   TSensorLuz = class ( Sensor )
     private
       _sala     : TList    ;  // lista de salas que ele mapeia
       _corredor : Corredor ;  // corredor que ele mapeia
     public
       constructor Create                                      ;
       destructor  Destroy                                     ; override ;
       procedure   adcionarSala( NovaSala : TSala )             ;
       procedure   adcionarCorredor( NovoCorredor : Corredor ) ;
       function    obterSala( numero : Integer )               : TSala     ;
       function    obterCorredor                               : TCorredor ;
       procedure   simularFalha                                ;
     end ; // TSensorLuz

implementation
// cria e inicializa sensor de luz
constructor TSensorLuz.Create ;
begin
   self._estado := 0 ;              // inicialmente detecta nenhuma iluminao
   self._falha := false ;           // inicialmente ele est funcionando
   self._sala := TList.Create( ) ;  // inicializa lista de sala
   self._corredor := nil ;          // inicializa corredor
   inherited Create( ) ;
end ; // SensorLuz.Create


// mtodo destrutor do sensor de luz
destructor TSensorLuz.Destroy ;
begin
   // no faz nada!!
   inherited Destroy( ) ;
end ; // SensorLuz.Destroy


// mtodo para colocar sensor de luz mapeando sala
procedure TSensorLuz.adcionarSala( NovaSala : TSala ) ;
begin
   _sala.Add( NovaSala ) ; // coloca mais uma sala na lista
end ; // SensorLuz.adcionarSala

// mtodo para colocar um sensor de luz mapeando seo de corredor
procedure TSensorLuz.adcionarCorredor( NovoCorredor : TCorredor ) ;
begin
   _corredor := NovoCorredor ; // sensor passa a mapear esta seo
end ; // SensorLuz.adcionarCorredor


// devolve objeto da classe Sala dado o seu ID que  mapeado por objeto da
// classe SensorLuz
function TSensorLuz.obterSala( numero : Integer ) : TSala ;
begin
// testa se nmero  vlido
   if ( ( numero > -1 )
    and ( numero < _sala.Count ) )
   then
   begin
      Result := _sala.Items[ numero ] ; // se  vlido, retorna a sala
   end
   else
   begin
       Result := nil ; // seno, retorna ponteiro invlido
   end ;
end ; // SensorLuz.obterSala

// devolve objeto da classe Corredor que  mapeado por objeto da classe
// SensorLuz
function TSensorLuz.obterCorredor : TCorredor ;
begin
   Result := _corredor ;
end ; // SensorLuz.obterCorredor


// permite falhar ou consertar um sensor externo de luz
// se est funcionando corretamente (_falha = false) vai ficar falhado
// se est falhado (_falha = true) vai passar a funcionar corretamente
procedure TSensorLuz.simularFalha ;
var
   i : Integer ;  // contador auxiliar para setar os grupos de luzes
begin
   _falha := NOT _falha ;  // inverte o indicador de falha

// s trata falha se sensor externo de luz extiver falhado
   if ( _falha = true )
   then
   begin
   // se h movimento na seo de corredor, seu grupo de luz deve estar aceso
      if ( ( _corredor.obterSensorMov( 1 ).obterEstado = 1 )
        or ( _corredor.obterSensorMov( 2 ).obterEstado = 1 )
        or ( _corredor.obterSensorMov( 3 ).obterEstado = 1 ) )
      then
      begin
         _corredor.acenderLuz( ) ; // acende grupo de luz de seo de corredor
      end ; // if

   // o nvel de iluminao padro em cada grupo de luz de sala deve ser de 100%
      for i := 0 to _sala.Count - 1 do
      begin
         Sala( _sala.Items[ i ] ).obterLuz( 1 ).definirInsPadrao( I_MAX_SALA ) ;
         Sala( _sala.Items[ i ] ).obterLuz( 2 ).definirInsPadrao( I_MAX_SALA ) ;
      end ; // for
   end ; // if

end ; // SensorLuz.simularFalha

end.
