unit Timer;

interface

uses Classes       ,
     Salas         ,
     Corredor      ,
     Constantes    ,
     Sensor        ,
     SensorMovimento;

Type
   TTimer = class ( TThread )
     private
        _T        : array[ 1..3 ] of Integer ;  // array de T's (T1, T2 e T3)
        _sala     : TSala                     ;  // sala mapeada pelo timer
        _corredor : Corredor                 ;  // corredor mapeado pelo timer
     protected
        constructor Create( NovaSala     : TSala    ;
                            NovoCorredor : TCorredor )  ;
        //destructor  Destroy                            ; override ;
        procedure   Execute                            ; override ;
        procedure   incTx( indice : Integer )          ;
        procedure   decTx( indice : Integer )          ;
        procedure   definirTx( indice : Integer ;
                               valor  : Integer  )     ;
        function    obterTx( indice : Integer )        : Integer ;
     public
        destructor  Destroy                            ; override ;
     end ; // TTimer

implementation
// cria e inicializa Timer
constructor TTimer.Create( NovaSala     : TSala    ;
                           NovoCorredor : TCorredor ) ;
begin
   _sala := NovaSala ;          // recebe sala a ser mapeada
   _corredor := NovoCorredor ;  // recebe corredor a ser mapeado
   _T[ 1 ] := 0 ;               // T1 inicialmente  zero minutos
   _T[ 2 ] := 0 ;               // T2 inicialmente  zero minutos
   _T[ 3 ] := 0 ;               // T3 inicialmente  zero minutos
   inherited Create( true ) ;   // aciona a execuo do Timer
end ; // TTimer.Create


// mtodo destrutor de Timer
destructor TTimer.Destroy ;
begin
   Terminate( ) ;         // serve para terminar o loop de execute do Timer
   inherited Destroy( ) ;
end ; // TTimer.Destroy


// executa o Timer (comea a temporizao)
procedure TTimer.Execute ;
var
   i : Integer ; // contador auxiliar
begin
// continua enquanto o temporizador no for destruido ou parado
   while( not Terminated ) do
   begin

      Sleep( TimeUnit ) ; // Aguarda por uma unidade de tempo

      for i := 1 to 3 do // decrementa T1, T2 e T3
      begin
         Dec( _T [ i ] ) ;   // decrementa 1 minuto
      end ; // for

   // s sinaliza T1 se T1 chegou a zero e se o timer  de sala
      if ( ( _T[ 1 ] = 0 )
       and ( _sala <> nil ) )
      then
      begin
         Synchronize( _sala.sinalizarT1 ) ; // sinaliza T1 na sala
      end ; // if

   // s sinaliza T2 se T2 chegou a zero e se o timer  de seo de corredor
      if ( ( _T[ 2 ] = 0 )
       and ( _corredor <> nil ) )
      then
      begin
         Synchronize( _corredor.sinalizarT2 ) ; // sinaliza T2 na seo
         Terminate( ) ;                         // termina o loop
      end ; // if

   // s sinaliza T3 se T3 chegou a zero e se o timer  de sala
      if ( ( _T[ 3 ] = 0 )
       and ( _sala <> nil ) )
      then
      begin
         Synchronize( _sala.sinalizarT3 ) ; // sinaliza T3 na sala
         Terminate( ) ;                     // termina o loop
      end ;

   // coloca em zero T1, T2 ou T3 se eles forem menores que zero
      for i := 1 to 3 do
      begin
         if ( _T[ i ] < 0 )
         then
         begin
            _T[ i ] := 0 ;
         end ; // if
      end ; // for

   end ; // while

end ; // TTimer.Execute


// incrementa em um minuto um dado Ti
procedure TTimer.incTx( indice : Integer ) ;
begin

// testa se indice est entre 1 e 3
   if ( ( indice > 0 )
    and ( indice < 4 ) )
   then
   begin
      Inc( _T[ indice ] ) ;  // incrementa Ti
   end ; // if

end ; // TTimer.incTx


// decrementa em um minuto um dado Ti
procedure TTimer.decTx( indice : Integer ) ;
begin

// testa se indice est entre 1 e 3
   if ( ( indice > 0 )
    and ( indice < 4 ) )
   then
   begin
      Dec( _T[ indice ] ) ;  // decrementa Ti
   end ; // if

end ; // TTimer.decTx


// define um Ti com um dado valor
procedure TTimer.definirTx( indice : Integer ;
                            valor  : Integer  ) ;
begin

// testa se indice est entre 1 e 3
   if ( ( indice > 0 )
    and ( indice < 4 ) )
   then
   begin

      if ( valor > 0 ) // testa se novo valor de Ti  positivo
      then
      begin
         _T[ indice ] := valor ; // se novo Ti > 0, Ti recebe novo Ti
      end // if then
      else
      begin
         _T[ indice ] := 0 ; // seno, Ti recebe zero
      end ; // if else

   end ; // if

end ; // TTimer.definirTx

// devolve um Ti
function TTimer.obterTx( indice : Integer ) : Integer ;
begin

// testa se indice est entre 1 e 3
   if ( ( indice > 0 )
    and ( indice < 4 ) )
   then
   begin
      result := _T[ indice ] ; // devolve o Ti se i  vlido
   end // if then
   else
   begin
      result := -1; // Erro se i  invlido
   end ; // if else

end ; // TTimer.obterTx

end.
